/*
** Module   : ColorTableEntry.java
** Abstract : a structure which hold RGB values.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 MAG 20140829 Created initial version.
** 002 MAG 20140911 Added asString method.
*/

package com.goldencode.p2j.ui;

import java.io.*;

/**
 * a structure which hold RGB values. 
 */
public class ColorRgb
implements Externalizable
{
   /** Red value */
   private int red;
   
   /** Green value */
   private int green;
   
   /** Blue value */
   private int blue;
   
   /** Color name */
   private String name;

   /**
    * Constructor.
.    */
   public ColorRgb() {
	   this(0,0,0);
   }
   /**
    * Constructor.
    * 
    * @param   red
    *          Red value.
    * @param   green
    *          Green value.
    * @param   blue
    *          Blue value.
    */
   public ColorRgb(int red, int green, int blue)
   {
      this.red = red;
      this.green = green;
      this.blue = blue;
   }

   /**
    * Constructor.
    * 
    * @param   red
    *          Red value.
    * @param   green
    *          Green value.
    * @param   blue
    *          Blue value.
    * @param   name
    *          Color name.
    */
   public ColorRgb(int red, int green, int blue, String name)
   {
      this(red, green, blue);
      this.name = name;
   }

   /**
    * Copy constructor.
    * 
    * @param   rgb
    *          ColorRgb copy.
    */
   public ColorRgb(ColorRgb rgb)
   {
      this(rgb.red, rgb.green, rgb.blue, rgb.name);
   }

   /**
    * Get red value.
    * 
    * @return  see above.
    */
   public int getRed()
   {
      return red;
   }

   /**
    * Set red value.
    * 
    * @param   red
    *          New red value
    */
   public void setRed(int red)
   {
      this.red = red;
   }

   /**
    * Get green value.
    * 
    * @return  see above.
    */
   public int getGreen()
   {
      return green;
   }

   /**
    * Set green value.
    * 
    * @param green
    *        New green value.
    */
   public void setGreen(int green)
   {
      this.green = green;
   }

   /**
    * Get blue value.
    * 
    * @return  see above.
    */
   public int getBlue()
   {
      return blue;
   }

   /**
    * Set blue value.
    * 
    * @param   blue
    *          New blue value.
    */
   public void setBlue(int blue)
   {
      this.blue = blue;
   }

   /**
    * Get a combination of red, green and blue value P4GL compliant.
    * 
    * @return  see above.
    */
   public int getRgb()
   {
      return this.red | (this.green << 8) | (this.blue << 16);
   }

   /**
    * Get a combination of red, green and blue value Java compliant.
    * 
    * @return  see above.
    */
   public int getGuiRgb()
   {
      return this.blue | (this.green << 8) | (this.red << 16);
   }

   /**
    * Set color values form a RGB value.
    * 
    * @param   rgb
    *          A combination of red, green and blue value.
    */
   public void setRgb(int rgb)
   {
      this.red   = rgb & 0xff;
      this.green = (rgb >>  8) & 0xff;
      this.blue  = (rgb >> 16) & 0xff;
   }

   /**
    * Get color name.
    * 
    * @return  see above.
    */
   protected String getName()
   {
      return name;
   }
   
   /**
    * Set color name.
    * 
    * @param   name
    *          Color name.
    */
   protected void setName(String name)
   {
      this.name = name;
   }

   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */   
   public void readExternal(ObjectInput in) 
   throws IOException, 
          ClassNotFoundException
   {
      red   = in.readInt();
      green = in.readInt();
      blue  = in.readInt();
   }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   public void writeExternal(ObjectOutput out) 
   throws IOException
   {
      out.writeInt(red);
      out.writeInt(green);
      out.writeInt(blue);      
   }
   
   /**
    * Return a RGB as string comma separated.
    * 
    * @return  see above.
    */
   public String asString()
   {
      return name == null ? red + "," + green  + "," + blue : name;
   }
}
